package cn.iocoder.yudao.module.product.listener;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.StrUtil;
import cn.iocoder.yudao.framework.apollo.core.event.export.ProductExcelExportPushEvent;
import cn.iocoder.yudao.framework.excel.util.ExcelUtils;
import cn.iocoder.yudao.module.infra.dal.dataobject.file.FileDO;
import cn.iocoder.yudao.module.infra.service.file.FileService;
import cn.iocoder.yudao.module.product.convert.product.ProductConvert;
import cn.iocoder.yudao.module.product.dal.dataobject.product.ProductAttrDO;
import cn.iocoder.yudao.module.product.dal.dataobject.product.ProductDO;
import cn.iocoder.yudao.module.product.dal.dataobject.product.ProductTypeDO;
import cn.iocoder.yudao.module.product.enums.AuditEnum;
import cn.iocoder.yudao.module.product.service.product.ProductAttrService;
import cn.iocoder.yudao.module.product.service.product.ProductService;
import cn.iocoder.yudao.module.product.service.product.ProductTypeService;
import cn.iocoder.yudao.module.product.vo.product.ProductExcelVO;
import cn.iocoder.yudao.module.product.vo.product.ProductExportReqVO;
import cn.iocoder.yudao.module.product.vo.productAttr.ProductAttrExportReqVO;
import cn.iocoder.yudao.module.product.vo.productType.ProductTypeExportReqVO;
import cn.iocoder.yudao.module.system.framework.ue.UeProperties;
import com.alibaba.fastjson.JSONObject;
import com.google.common.base.Joiner;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.context.event.EventListener;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.io.File;
import java.io.FileInputStream;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import static cn.iocoder.yudao.framework.excel.constant.ExportConstant.DATA_FORMAT;


/**
 * 自编号订单excel导出监听
 *
 * @author zhengYi
 */
@Component("ProductExcelExportListener")
@AllArgsConstructor
@Slf4j
public class ProductExcelExportListener {

    private UeProperties ueProperties;
    private FileService fileService;

    @Resource
    private ProductService productService;

    @Resource
    private ProductTypeService productTypeService;
    @Resource
    private ProductAttrService productAttrService;


    /**
     * 订单导出监听
     *
     * @param event 编号记录
     */
    @EventListener(ProductExcelExportPushEvent.class)
    public void productExcelExportPushEvent(ProductExcelExportPushEvent event) {
        if (StringUtils.isNotBlank(event.getRequestParams())) {
            try {

                DateTimeFormatter formatter = DateTimeFormatter.ofPattern(DATA_FORMAT);
                String nowTime = formatter.format(LocalDateTime.now());
                String dir = ueProperties.getTempDir().concat("/product/excel/");
                File fileDir = new File(dir);
                if(!fileDir.exists()) {
                    fileDir.mkdirs();
                }
                String fileName = event.getUserId().toString().concat(StrUtil.DASHED).concat(event.getUserType().toString()).concat(StrUtil.DASHED).concat(nowTime).concat("product.xlsx");

                ProductExportReqVO exportReqVO = JSONObject.parseObject(event.getRequestParams(), ProductExportReqVO.class);
                exportReqVO.setFilter(false);
                List<ProductDO> list = productService.getProductList(exportReqVO);

                if(CollectionUtil.isEmpty(list)) return;

                List<ProductAttrDO> attrList = productAttrService.getProductAttrList(new ProductAttrExportReqVO());
                List<ProductTypeDO> typeList = productTypeService.getProductTypeList(new ProductTypeExportReqVO());

                Map<Long, ProductAttrDO> attrMap = attrList.stream()
                        .collect(Collectors.toMap(ProductAttrDO::getId, v -> v));
                Map<Long, ProductTypeDO> typeMap = typeList.stream()
                        .collect(Collectors.toMap(ProductTypeDO::getId, v -> v));

                // 导出 Excel
                List<ProductExcelVO> datas = ProductConvert.INSTANCE.convertList02(list);

                for (ProductExcelVO excelVO : datas) {
                    excelVO.setShowId(excelVO.getId().toString());
                    Long typeId = excelVO.getTypeId();
                    ProductTypeDO typeDO = typeMap.get(typeId);
                    if(typeDO != null) {
                        excelVO.setTypeName(typeDO.getTitleZh());
                    }
                    String attrId = excelVO.getAttrId();
                    if(StringUtils.isNotEmpty(attrId)) {
                        String[] attrArray = attrId.split(",");
                        List<String> attrNameList = new ArrayList<>();
                        for (String attr : attrArray) {
                            Long id = Long.valueOf(attr);
                            ProductAttrDO attrDO = attrMap.get(id);
                            if(attrDO != null) {
                                attrNameList.add(attrDO.getAttrName());
                            }
                        }
                        excelVO.setAttrName(Joiner.on(",").join(attrNameList));
                    }
                    excelVO.setAuditStatusName(AuditEnum.getTextByStatus(excelVO.getAuditStatus()));
                }

                String path = ExcelUtils.write(dir, fileName, "数据", ProductExcelVO.class, datas);

                // 获取到临时文件
                File file = new File(path);
                // 创建FileInputStream对象
                FileInputStream fileInputStream = new FileInputStream(file);
                // 读取文件内容
                byte[] fileBytes = new byte[(int) file.length()];
                fileInputStream.read(fileBytes);
                // 关闭文件流
                fileInputStream.close();
                // 将文件上传到资源服务器
                FileDO fileDO = fileService.createFile(dir, fileName, fileBytes);
                event.setPath(fileDO.getPath());
                event.setFileName(fileDO.getPath());
                event.setUrl(fileDO.getUrl());
                event.setFileId(fileDO.getId());
            } catch (Exception e) {
                // TODO 测试阶段打印堆栈错误信息，便于分析原因
                e.printStackTrace();
                event.setResult(e.getMessage());
            }
        } else {
            event.setResult("param fail");
        }

    }
}
